@@ -15,13 +15,14 @@ heroku3.py
 
 # Meta.
 
-__title__ = 'heroku3'
-__author__ = 'Heroku'
-__copyright__ = 'Copyright 2013 Heroku'
-__license__ = 'MIT'
+__title__ = "heroku3"
+__author__ = "Heroku"
+__copyright__ = "Copyright 2013 Heroku"
+__license__ = "MIT"
 
-__version__ = '3.4.0'
+__version__ = "4.2.3"
 
 # Module namespace.
 
-from .core import from_key
+# Project libraries
+from .core import from_key  # noqa
